#include <math.h>
#include <mex.h>
#include "asa_user.h"
#include "asa_cg.h"

#define N_x 0
#define N_lo 1
#define N_hi 2
#define N_f 3
#define N_g 4
#define N_fg 5
#define N_hs 6
#define N_gt 7
#define N_cgp 8
#define N_asap 9

double myvalue(asa_objective *asa);
void mygrad(asa_objective *asa);
double myvalgrad(asa_objective *asa);
void myHessSolve(asa_objective *asa);
// void asa_printcgParms(asacg_parm  *Parm);
// void asa_printParms(asa_parm  *Parm);

mxArray *LHS_f[1];
mxArray *RHS_f[2];
mxArray *LHS_grad[1];
mxArray *RHS_grad[2];
mxArray *LHS_fg[2];
mxArray *RHS_fg[2];
mxArray *LHS_hs[1];
mxArray *RHS_hs[6];


void mexFunction( int nlhs, mxArray *plhs[], int nrhs, const mxArray*prhs[] ){
	int n, i, j; 
	double *tmp_x, *x, *lo, *hi;
	const char **fname_cg;
	const char **fname_asap;
	int ifield_cg, nfields_cg;
	int ifield_asap, nfields_asap;
	double val_cg, val_asap;
	double *vals;
	double grad_tol;
	mwIndex idx;

	idx = 0;
    /* if you want to change parameter value, you need the following: */
    asacg_parm cgParm;
    asa_parm asaParm;

	n = (int)mxGetM(prhs[N_x]);
	x = mxGetPr(prhs[N_x]);
	lo = mxGetPr(prhs[N_lo]);
	hi = mxGetPr(prhs[N_hi]);

	RHS_f[0] = mxDuplicateArray(prhs[N_f]);
	RHS_f[1] = mxCreateDoubleMatrix(n,1,mxREAL);

	RHS_grad[0] = mxDuplicateArray(prhs[N_g]);
	RHS_grad[1] = mxCreateDoubleMatrix(n,1,mxREAL);


	if((nrhs >= 6) && !mxIsEmpty(prhs[N_fg])){
    	RHS_fg[0] = mxDuplicateArray(prhs[N_fg]);
		RHS_fg[1] = mxCreateDoubleMatrix(n,1,mxREAL);
    }

    if ((nrhs >= 7) && !mxIsEmpty(prhs[N_hs])){
	    RHS_hs[0] = mxDuplicateArray(prhs[N_hs]);
	    RHS_hs[1] = mxCreateDoubleMatrix(n,1,mxREAL);
	    RHS_hs[2] = mxCreateDoubleMatrix(n,1,mxREAL);
	    // RHS_hs[3] = mxCreateDoubleMatrix(1,1,mxREAL);
  	  	// RHS_hs[4] = mxCreateNumericArray(1,1,mxINT32_CLASS);        
	    RHS_hs[5] = mxCreateNumericMatrix(n,1,mxINT64_CLASS,mxREAL);       
    }

	if ((nrhs >= 8) && (!mxIsEmpty(prhs[N_gt])) ){
		grad_tol = mxGetScalar(prhs[N_gt]);
	}else{
		grad_tol = 1.e-8;
	}
// 	printf("grad_tol=%.10e\n", grad_tol);

	 /* if you want to change parameter value, initialize strucs with default */
    asa_cg_default(&cgParm);
    asa_default(&asaParm);

    /* if you want to change parameters, change them here: */
    if ((nrhs>=9) && (!mxIsEmpty(prhs[N_cgp]))){
    	nfields_cg = mxGetNumberOfFields(prhs[N_cgp]);
//     	printf("------- %d params are reset by matlab code -----------\n", nfields_cg);
    	fname_cg = mxCalloc(nfields_cg, sizeof(*fname_cg));
    	for(ifield_cg=0;ifield_cg<nfields_cg;ifield_cg++){
    		fname_cg[ifield_cg] = mxGetFieldNameByNumber(prhs[N_cgp], ifield_cg);
    		vals = mxGetPr( mxGetField(prhs[N_cgp], idx, fname_cg[ifield_cg]) );
    		val_cg = vals[0];
    		if (strcmp(fname_cg[ifield_cg], "PrintParms")==0){ // int
	    		if ((int)val_cg == 1){
	    			cgParm.PrintParms = TRUE;
	    		}else{
	    			cgParm.PrintParms = FALSE;
	    		}
// 	    		printf("%s=%d\n", fname_cg[ifield_cg], cgParm.PrintParms);
	    	}else if (strcmp(fname_cg[ifield_cg], "PrintLevel")==0){ // int
	    		cgParm.PrintLevel = (int)val_cg;
// 	    		printf("%s=%d\n", fname_cg[ifield_cg], cgParm.PrintLevel);
	    	}else if (strcmp(fname_cg[ifield_cg], "LBFGS")==0){ // int
	    		if ( (int)val_cg == 1){
	    			cgParm.LBFGS = TRUE;
	    		}else{
	    			cgParm.LBFGS = FALSE;
	    		}
// 	    		printf("%s=%d\n", fname_cg[ifield_cg], cgParm.LBFGS);
	    	}else if (strcmp(fname_cg[ifield_cg], "memory")==0){ //int
	    		cgParm.memory = (int)val_cg;
// 	    		printf("%s=%d\n", fname_cg[ifield_cg], cgParm.memory);
	    	}else if (strcmp(fname_cg[ifield_cg], "PertRule")==0){ //int
	    		if ((int)val_cg == 1){
	    			cgParm.PertRule = TRUE;
	    		}else{
	    			cgParm.PertRule = FALSE;
	    		}
// 	    		printf("%s=%d\n", fname_cg[ifield_cg], cgParm.PertRule);
	    	}else if (strcmp(fname_cg[ifield_cg], "qrestart")==0){
	    		cgParm.qrestart = (int)val_cg;
// 	    		printf("%s=%d\n", fname_cg[ifield_cg], cgParm.qrestart);
	    	}
		}
    }
    // if (cgParm.PrintParms) asa_printcgParms(&cgParm);


    if ((nrhs>=10) && (!mxIsEmpty(prhs[N_asap]))){
    	nfields_asap = mxGetNumberOfFields(prhs[N_asap]);
//     	printf("------- %d params are reset by matlab code -----------\n", nfields_asap);
    	fname_asap = mxCalloc(nfields_asap, sizeof(*fname_asap));
    	for(ifield_asap=0;ifield_asap<nfields_asap;ifield_asap++){
    		fname_asap[ifield_asap] = mxGetFieldNameByNumber(prhs[N_asap], ifield_asap);
    		vals = mxGetPr( mxGetField(prhs[N_asap], idx, fname_asap[ifield_asap]) );
    		val_asap = vals[0];
    		if (strcmp(fname_asap[ifield_asap], "PrintParms")==0){
	    		if ((int)val_asap == 1){
	    			asaParm.PrintParms = TRUE;
	    		}else{
	    			asaParm.PrintParms = FALSE;
	    		}
// 	    		printf("%s=%d\n", fname_asap[ifield_asap], asaParm.PrintParms);
	    	}else if (strcmp(fname_asap[ifield_asap], "PrintLevel")==0){
	    		asaParm.PrintLevel = (int)val_asap;
// 	    		printf("%s=%d\n", fname_asap[ifield_asap], asaParm.PrintLevel);
	    	}else if (strcmp(fname_asap[ifield_asap], "PrintFinal")==0){
	    		if ((int)val_asap == 1){
	    			asaParm.PrintFinal = TRUE;
	    		}else{
	    			asaParm.PrintFinal = FALSE;
	    		}
// 	    		printf("%s=%d\n", fname_asap[ifield_asap], asaParm.PrintFinal);
	    	}else if (strcmp(fname_asap[ifield_asap], "PertRule")==0){
	    		if ((int)val_asap == 1){
	    			asaParm.PertRule = TRUE;
	    		}else{
	    			asaParm.PertRule = FALSE;
	    		}
// 	    		printf("%s=%d\n", fname_asap[ifield_asap], asaParm.PertRule);
	    	}else if (strcmp(fname_asap[ifield_asap], "GradProjOnly")==0){
	    		if ((int)val_asap == 1){
	    			asaParm.GradProjOnly = TRUE;
	    		}else{
	    			asaParm.GradProjOnly = FALSE;
	    		}
// 	    		printf("%s=%d\n", fname_asap[ifield_asap], asaParm.GradProjOnly);
	    	}else if (strcmp(fname_asap[ifield_asap], "HardConstraint")==0){
	    		if ((int)val_asap == 1){
	    			asaParm.HardConstraint = TRUE;
	    		}else{
	    			asaParm.HardConstraint = FALSE;
	    		}
// 	    		printf("%s=%d\n", fname_asap[ifield_asap], asaParm.HardConstraint);
	    	}else if (strcmp(fname_asap[ifield_asap], "lmin")==0){
	    		asaParm.lmin = val_asap;
// 	    		printf("%s=%.10e\n", fname_asap[ifield_asap], asaParm.lmin);
	    	}else if (strcmp(fname_asap[ifield_asap], "lmax")==0){
	    		asaParm.lmax = val_asap;
// 	    		printf("%s=%.10e\n", fname_asap[ifield_asap], asaParm.lmax);
	    	}
		}
    }
    // if (asaParm.PrintParms) asa_printParms(&asaParm);

    /* run the code */
    if ((nrhs >= 6) && (nrhs<=10)){

    	if (nrhs == 6){
    		if (mxIsEmpty(prhs[N_fg])){
    			asa_cg (x, lo, hi, n, NULL, &cgParm, &asaParm, 
    				grad_tol, myvalue, mygrad, NULL, NULL, NULL, NULL);
    		}else{
    			asa_cg (x, lo, hi, n, NULL, &cgParm, &asaParm, 
    				grad_tol, myvalue, mygrad, myvalgrad, NULL, NULL, NULL);
    		}
    	}else{

	    	if ( (mxIsEmpty(prhs[N_fg])) && (mxIsEmpty(prhs[N_hs]))){
	    		asa_cg (x, lo, hi, n, NULL, &cgParm, &asaParm, 
					grad_tol, myvalue, mygrad, NULL, NULL, NULL, NULL);
	    	}else if (mxIsEmpty(prhs[N_fg])){
	    		asa_cg (x, lo, hi, n, NULL, &cgParm, &asaParm, 
					grad_tol, myvalue, mygrad, NULL, myHessSolve, NULL, NULL);
	    	}else if (mxIsEmpty(prhs[N_hs])){
				asa_cg (x, lo, hi, n, NULL, &cgParm, &asaParm,
					grad_tol, myvalue, mygrad, myvalgrad, NULL, NULL, NULL);
	    	}else{      
	    		asa_cg (x, lo, hi, n, NULL, &cgParm, &asaParm,
					grad_tol, myvalue, mygrad, myvalgrad, myHessSolve, NULL, NULL);
	    	}
	    }

	}else{
		mexErrMsgTxt("calling format\n: x=asacg_wrapper(x0, lo, hi, myvalue, mygrad, myvalgrad, myHessSolve, grad_tol, cgParm, asaParm)\n");
	}


    /* output the optimal values*/
	plhs[0] = mxCreateDoubleMatrix(n,1,mxREAL);
	tmp_x = mxGetPr(plhs[0]);
	for(i=0;i<n;i++)
		tmp_x[i] = x[i];
}

double myvalue(asa_objective *asa){
	INT i, n;
	double *x, f;

	n = asa->n;
	x = (double *) mxGetPr(RHS_f[1]);
	for(i=0;i<n;i++)
		x[i] = asa->x[i];
	mexCallMATLAB(1, LHS_f, 2, RHS_f, "feval");
	f = mxGetScalar(LHS_f[0]);
	return f;
}

void mygrad(asa_objective *asa){
	INT i, n;
	double *x, *g, *tmp_g;

	n = asa->n;
	x = (double *) mxGetPr(RHS_grad[1]);
	for(i=0;i<n;i++)
		x[i] = asa->x[i];
	mexCallMATLAB(1, LHS_grad, 2, RHS_grad, "feval");

	g = asa->g;
	tmp_g = mxGetPr(LHS_grad[0]);
	for(i=0;i<n;i++)
		g[i] = tmp_g[i];
}

double myvalgrad(asa_objective *asa){
	INT i, n;
	double *x, *g, *tmp_g, f;

	n = asa->n;
	x = (double *) mxGetPr(RHS_fg[1]);
	for(i=0;i<n;i++)
		x[i] = asa->x[i];
	mexCallMATLAB(2, LHS_fg, 2, RHS_fg, "feval");

	f = mxGetScalar(LHS_fg[0]);

	g = asa->g;
	tmp_g = mxGetPr(LHS_fg[1]);
	for(i=0;i<n;i++)
		g[i] = tmp_g[i];
	return f;
}

void myHessSolve(asa_objective *asa){
	INT i, n, nfree, *ifree;
        int Hinfo;
	double *x, *b, *d, *tmp_d, Hscale;
       

	n = asa->n;
	x = (double *) mxGetPr(RHS_hs[1]);
	for(i=0;i<n;i++)
		x[i] = asa->x[i];

	nfree = asa->nfree;
	b = (double *) mxGetPr(RHS_hs[2]);
	for(i=0;i<nfree;i++)
		b[i] = asa->b[i];
     
    // Hscale = mxGetPr(RHS_hs[3]);
    // Hscale = asa->Hscale;
    RHS_hs[3] = mxCreateDoubleScalar(asa->Hscale);

    // Hinfo  = mxGetPr(RHS_hs[4]);
    // Hinfo  = asa->Hinfo;
    RHS_hs[4] = mxCreateDoubleScalar(asa->Hinfo);

	ifree = (INT *) mxGetData(RHS_hs[5]);
    if ( nfree < n ) // assume ifree = zeros(n,1) when n = nfree
    {
	    for(i=0;i<nfree;i++)
		    ifree[i] = asa->ifree[i] + 1; // index in MATLAB starts from 1
    }
	
	// We want the other indices to be zero
	for(i=nfree;i<n;i++)
		ifree[i] = 0;

	mexCallMATLAB(1, LHS_hs, 6, RHS_hs, "feval");

	d = asa->d;
	tmp_d = mxGetPr(LHS_hs[0]);
	for(i=0;i<nfree;i++)
		d[i] = tmp_d[i];
}