function test_asacg
n = 100;
lo = zeros(n,1);
hi = ones(n,1);

cgParm.PrintParms = 0;%true;
cgParm.PrintLevel = 1 ;
cgParm.LBFGS = 0; %false ;
cgParm.memory = 11 ;
cgParm.PertRule = 1;%true ;
cgParm.qrestart = 6 ;

asaParm.PrintParms = 0; %true ;
asaParm.PrintLevel = 0 ;
asaParm.PrintFinal = 1; %true;
asaParm.PertRule = 1; %true ;
asaParm.GradProjOnly = 0; %false ;
asaParm.HardConstraint = 1; %true ;
asaParm.lmin = 1.0e-20 ;
asaParm.lmax = 1.0e+20 ;

grad_tol = 1e-7;
% the first six parameters are reqiured, 
% while the rest four parameters are optional as either empty or missing

x0 = ones(n,1);
hess_solver_f = @(x,b,Hscale,Hinfo,ifree)myHessSolve(x,b,Hscale,Hinfo,ifree);
x = asacg_wrapper(x0, lo, hi, @(x)myvalue(x), @(x)mygrad(x), @(x)myvalgrad(x), hess_solver_f, grad_tol, cgParm, asaParm);
f = myvalue(x)

% x0 = ones(n,1);
% x = asacg_wrapper(x0, lo, hi, @(x)myvalue(x), @(x)mygrad(x), @(x)myvalgrad(x), [], grad_tol, cgParm, asaParm);
% f = myvalue(x)
% 
% x0 = ones(n,1);
% x = asacg_wrapper(x0, lo, hi, @(x)myvalue(x), @(x)mygrad(x), @(x)myvalgrad(x), [], [], cgParm, asaParm);
% f = myvalue(x)
% 
% x0 = ones(n,1);
% x = asacg_wrapper(x0, lo, hi, @(x)myvalue(x), @(x)mygrad(x), [], [], grad_tol, [], []);
% f = myvalue(x)
% 
% x0 = ones(n,1);
% x = asacg_wrapper(x0, lo, hi, @(x)myvalue(x), @(x)mygrad(x), [], [], grad_tol);
% f = myvalue(x)
% 
% x0 = ones(n,1);
% x = asacg_wrapper(x0, lo, hi, @(x)myvalue(x), @(x)mygrad(x), []);
% f = myvalue(x)
% 
% x0 = ones(n,1);
% x = asacg_wrapper(x0, lo, hi, @(x)myvalue(x), @(x)mygrad(x), @(x)myvalgrad(x));
% f = myvalue(x)


function f=myvalue(x)
n = length(x);
f=0;
for i=1:n
    t = sqrt(i);
    f = f + exp(x(i)) - t * x(i);
end

function grad=mygrad(x)
n = length(x);
grad = zeros(n,1);
for i=1:n
    t = sqrt(i);
    grad(i) = exp(x(i)) - t;
end

function [f, grad]=myvalgrad(x)
n = length(x);
f=0;
grad = zeros(n,1);
for i=1:n
    t = sqrt(i);
    f = f + exp(x(i)) - t * x(i);
    grad(i) = exp(x(i)) - t;
end

%   Hinfo = 1 => evaluate Hess at x and solve A*d = b;
%   Hinfo = 2 => use previous Hess and solve A*d = b ;
%   Hinfo = 3 => evaluate Hess at x and calculate d = A*b;
%   Hinfo = 4 => use previous Hess and calculate d = A*b ;                
%   where  A = Hess(ifree,ifree) + Hscale I, with Hscale >= 0.

function d=myHessSolve(x,b,Hscale,Hinfo,ifree)
global Hess;

n = length(x);
nfree = find(ifree,1,'last');

if isempty(nfree)  % full dimension
    nfree = n;  
    ifree = [1:n]';
else               % dimension = nfree
    ifree = ifree(1:nfree);
    b = b(1:nfree); 
end

if Hinfo == 1
    % generate Hessian at x
    Hess = speye(n);
    for i=1:n
        Hess(i,i) = exp(x(i));
    end
    % form A = Hess(ifree,ifree) + Hscale I
    if Hscale > 0
        A = Hess(ifree,ifree)+Hscale*speye(nfree);
    else
        A = Hess(ifree,ifree);
    end
   % solve A*d = b
   d = A\b; 
elseif Hinfo == 2       
    % form A = Hess(ifree,ifree) + Hscale I
    if Hscale > 0
        A = Hess(ifree,ifree)+Hscale*speye(nfree);
    else
        A = Hess(ifree,ifree);
    end
   % solve A*d = b
   d = A\b;
elseif Hinfo == 3
    % generate Hessian at x
    Hess = speye(n);
    for i=1:n
        Hess(i,i) = exp(x(i));
    end
    % calculate d = A*b, where A = Hess(ifree,ifree) + Hscale I
    if Hscale > 0
        d = Hess(ifree,ifree)*b;
    else
        d = Hess(ifree,ifree)*b + Hscale*b; 
    end

else   % Hinfo = 4 
    % calculate d = A*b, where A = Hess(ifree,ifree) + Hscale I
     if Hscale > 0
        d = Hess(ifree,ifree)*b;
    else
        d = Hess(ifree,ifree)*b + Hscale*b; 
    end  
end
